#ifndef _INCLUDE_CRCF_H_
#define _INCLUDE_CRCF_H_

#include <stdbool.h>

#define CRCF_ID_SIZE 37
#define CRCF_NAME_SIZE 101
#define CRCF_NAME_LEN 100

#define CRCF_ID_BROADCAST "00000000-0000-0000-0000-000000000000"

enum crcf_actor {
	CRCF_ACTOR_UNKNOWN,
	CRCF_ACTOR_LOCO,
	CRCF_ACTOR_DECODER,
	CRCF_ACTOR_PROTOCOL,
	CRCF_ACTOR_FUNCTION,
};

/* Each db entry starts with a crcf id */
struct crcf_id {
	char id[CRCF_ID_SIZE];
	enum crcf_actor actor;
};

struct crcf_loco {
	struct crcf_id id;
	
	char name[CRCF_NAME_SIZE];
	
	struct crcf_id decoder;
};

struct crcf_decoder {
	struct crcf_id id;
	
	struct crcf_id loco;
	struct crcf_id protocol;
};

struct crcf_protocol {
	struct crcf_id id;

	int address;
	int bus;
	int speedsteps;
	struct crcf_id decoder;
};

struct crcf_function {
	struct crcf_id id;
	
	int nr;
	int pushbutton;
	
	struct crcf_id protocol;
};

char *crcf_str_encode(char *in);
char *crcf_str_decode(char *in);

bool crcf_id_broadcast(char *id);

struct crcf_id *crcf_db_search_id(char *id);
struct crcf_id *crcf_db_search_id_actor(char *id, enum crcf_actor actor);
struct crcf_id *crcf_db_iterate(struct crcf_id *prev);

struct crcf_loco *crcf_loco_create(char *id);
int crcf_loco_name_set(struct crcf_loco *loco, char *name);
char *crcf_loco_name_get(struct crcf_loco *loco);
int crcf_loco_decoder_set(struct crcf_loco *loco, struct crcf_decoder *decoder);

struct crcf_decoder *crcf_decoder_create(char *id);
int crcf_decoder_protocol_set(struct crcf_decoder *decoder, struct crcf_protocol *protocol);
int crcf_decoder_loco_set(struct crcf_decoder *decoder, struct crcf_loco *loco);
struct crcf_loco *crcf_decoder_loco_get(struct crcf_decoder *decoder);

struct crcf_protocol *crcf_protocol_create(char *id);
int crcf_protocol_address_set(struct crcf_protocol *protocol, int address);
int crcf_protocol_bus_set(struct crcf_protocol *protocol, int bus);
int crcf_protocol_speedsteps_set(struct crcf_protocol *protocol, int speedsteps);
int crcf_protocol_decoder_set(struct crcf_protocol *protocol, struct crcf_decoder *decoder);
struct crcf_decoder *crcf_protocol_decoder_get(struct crcf_protocol *protocol);
struct crcf_protocol *crcf_db_search_protocol(int bus, int address);
int crcf_db_search_protocol_functioncount(struct crcf_protocol *protocol);
struct crcf_function *crcf_db_iterate_protocol_function(struct crcf_protocol *protocol, struct crcf_function *prev);
struct crcf_function *crcf_db_protocol_search_function(struct crcf_protocol *protocol, int number);

struct crcf_function *crcf_function_create(char *id);
int crcf_function_number_set(struct crcf_function *function, int nr);
int crcf_function_number_get(struct crcf_function *function);
int crcf_function_mode_set(struct crcf_function *function, int pushbutton);
int crcf_function_mode_get(struct crcf_function *function);
int crcf_function_protocol_set(struct crcf_function *function, struct crcf_protocol *protocol);
struct crcf_protocol *crcf_function_protocol_get(struct crcf_function *function);

#endif /* _INCLUDE_CRCF_H_ */
