#define _BSD_SOURCE
#include "display_sdl.h"

#include <SDL/SDL.h>
#include <unistd.h>
#include <pthread.h>

static SDL_Surface *screen;
static uint32_t *pixels;
static uint32_t white, red, blue, green;
static uint32_t black = 0;
static int display_w, display_h;
static int pipefd[2];

static void *event_loop(void *arg)
{
	SDL_Event event;
	
	while (1) {
	        while(SDL_PollEvent(&event))
		{
			/* If a quit event has been sent */
			if (event.type == SDL_QUIT) {
				SDL_Surface *tmp = screen;
				
				screen = NULL;
				
				printf("event quit\n");
				if (tmp)
					SDL_FreeSurface(tmp);
	
				SDL_Quit();
				
				return NULL;
			}
			if (event.type == SDL_KEYDOWN) {
				char *c = SDL_GetKeyName(event.key.keysym.sym);
				
				write(pipefd[1], c, strlen(c));
			}
			if (event.type == SDL_VIDEOEXPOSE) {
				SDL_Flip(screen);
			}
		}
		usleep(100000);
	}
	
	return NULL;
}

int display_sdl_init(int w, int h)
{
	pthread_t thread;
	
	SDL_Init(SDL_INIT_EVERYTHING);
	
	screen = SDL_SetVideoMode(w * 2, h * 2, 32, SDL_SWSURFACE);
	if (!screen) {
		printf("Could not create screen\n");
		return -1;
	}

	SDL_WM_SetCaption("srcp-joystick", NULL);

	green = screen->format->Gmask;
	blue = screen->format->Bmask;
	red = screen->format->Rmask;
	white = green | blue | red;

	display_w = w;
	display_h = h;

	pixels = screen->pixels;

	/* Set screen to black*/
	SDL_FillRect(screen, NULL, black);
	SDL_Flip(screen);
	
	pipe(pipefd);
	
	pthread_create(&thread, NULL, event_loop, NULL);
	
	return pipefd[0];
}

int display_sdl_term(void)
{
	if (screen)
		SDL_FreeSurface(screen);
	
	SDL_Quit();
	
	return 0;
}

int display_sdl_update(uint8_t *buffer)
{
	int x, y;

	if (!screen)
		return -1;
	
	for (x = 0; x < display_w; x++) {
		for (y = 0; y < display_h; y++) {
			uint32_t col;
		
			if (buffer[x + y * display_w])
				col = black;
			else
				col = green;
	
			pixels[(y * display_w * 2 + x) * 2 + 0] = col;
			pixels[(y * display_w * 2 + x) * 2 + 1] = col;
			pixels[(y * display_w * 2 + x + display_w) * 2 + 0] = col;
			pixels[(y * display_w * 2 + x + display_w) * 2 + 1] = col;
		}
	}
	SDL_Flip(screen);

	return 0;
}

