#include <stdio.h>
#include <stdlib.h>
#include <termios.h>
#include <unistd.h>
#include "srcp.h"

static void srcp_console_status(struct srcp_loc_dcc *loc)
{
	printf("%s %03d/%03d %c%c%c%c%c\n",
	    loc->drivemode == SRCP_LOC_DCC_DRIVEMODE_FORWARD ? "Forward " : "Backward",
	    loc->V, loc->V_max,
	    loc->functions[0] ? 'o' : '.',
	    loc->functions[1] ? 'o' : '.',
	    loc->functions[2] ? 'o' : '.',
	    loc->functions[3] ? 'o' : '.',
	    loc->functions[4] ? 'o' : '.'
	);
}

int main(int argc, char **argv)
{
	struct srcp_loc_dcc loc;
	struct srcp *srcp;
	int response;
	
	if (argc < 11) {
		printf("Usage: %s [host] [address] [dir] [speed] [speed_max] [f0] [f1] [f2] [f3] [f4]\n",
		    argv[0]);
		return 1;
	}
	
	srcp = srcp_create(argv[1], "4303");
	if (!srcp) {
		perror("srcp_create()");
		return 1;
	}

	srcp_go(srcp);

	loc.bus = 1;
	loc.address = atoi(argv[2]);
	if (loc.address > 127)
		loc.protocolversion = SRCP_LOC_DCC_PROTOCOLVERSION_LONG;
	else
		loc.protocolversion = SRCP_LOC_DCC_PROTOCOLVERSION_SHORT;

	response = srcp_loc_dcc_get(srcp, &loc);
	if (SRCP_RESPONSE_ERROR(response)) {
		loc.decoderspeedsteps = 128;
		loc.nrofdecoderfunctions = 5;
		printf("Initializing new loc\n");
		srcp_loc_dcc_init(srcp, &loc);
	} else {
		printf("Loc already known, old values:\n");
		srcp_console_status(&loc);
	}

	loc.drivemode = atoi(argv[3]);
	loc.V = atoi(argv[4]);
	loc.V_max = atoi(argv[5]);
	loc.functions[0] = atoi(argv[6]);
	loc.functions[1] = atoi(argv[7]);
	loc.functions[2] = atoi(argv[8]);
	loc.functions[3] = atoi(argv[9]);
	loc.functions[4] = atoi(argv[10]);
	
	srcp_loc_dcc_set(srcp, &loc);
	printf("New values:\n");
	srcp_console_status(&loc);

	return 0;
}
