#include <stdio.h>
#include <stdlib.h>
#include <termios.h>
#include <unistd.h>
#include "srcp.h"

static void srcp_console_status(struct srcp_loc_dcc *loc)
{
	printf("%s %03d/%03d %c%c%c%c%c\n",
	    loc->drivemode == SRCP_LOC_DCC_DRIVEMODE_FORWARD ? " Forward" : "Backward",
	    loc->V, loc->V_max,
	    loc->functions[0] ? 'o' : '.',
	    loc->functions[1] ? 'o' : '.',
	    loc->functions[2] ? 'o' : '.',
	    loc->functions[3] ? 'o' : '.',
	    loc->functions[4] ? 'o' : '.'
	);
}

int main(int argc, char **argv)
{
	struct srcp_loc_dcc loc;
	struct srcp *srcp;
	struct termios oldtio,newtio;
	
	if (argc < 3) {
		printf("Usage %s [host] [loc address]\n", argv[0]);
		return -1;
	}
	
	printf("Opening connection to %s for loc %s\n", argv[1], argv[2]);
	
	srcp = srcp_create(argv[1], "4303");
	if (!srcp) {
		perror("srcp_create()");
		return 1;
	}

	loc.bus = 1;
	loc.address = atoi(argv[2]);
	printf("Using loc address: %d\n", loc.address);

	loc.drivemode = SRCP_LOC_DCC_DRIVEMODE_FORWARD;
	loc.decoderspeedsteps = 128;
	loc.V = 0;
	loc.V_max = 126;
	loc.nrofdecoderfunctions = 5;
	loc.functions[0] = true;
	loc.functions[1] = false;
	loc.functions[2] = false;
	loc.functions[3] = false;
	loc.functions[4] = false;
	
	srcp_go(srcp);
	
	tcgetattr(0, &oldtio);
	newtio = oldtio;
	newtio.c_iflag &= ~(IGNBRK | BRKINT | PARMRK | ISTRIP
                | INLCR | IGNCR | ICRNL | IXON);
	newtio.c_lflag &= ~(ECHO | ICANON | ISIG | IEXTEN);
	newtio.c_cflag &= ~(CSIZE | PARENB);
	newtio.c_cflag |= CS8;
	tcsetattr(0, TCSANOW, &newtio);

	srcp_loc_dcc_init(srcp, &loc);

	while (1) {
		int in;

		srcp_console_status(&loc);
		srcp_loc_dcc_set(srcp, &loc);
		in = fgetc(stdin);

		switch (in) {
			case '.':
				loc.V++;
				if (loc.V > 126)
					loc.V = 126;
				break;
			case ',':
				loc.V--;
				if (loc.V < 0)
					loc.V = 0;
				break;
			case 'd':
				loc.drivemode = 
				    loc.drivemode == SRCP_LOC_DCC_DRIVEMODE_FORWARD ?
				    SRCP_LOC_DCC_DRIVEMODE_BACKWARD :
				    SRCP_LOC_DCC_DRIVEMODE_FORWARD;

			case 'h':
				loc.V = 0;
				break;
			case '0':
				loc.functions[0] = !loc.functions[0];
				break;
			case '1':
				loc.functions[1] = !loc.functions[1];
				break;
			case '2':
				loc.functions[2] = !loc.functions[2];
				break;
			case '3':
				loc.functions[3] = !loc.functions[3];
				break;
			case '4':
				loc.functions[4] = !loc.functions[4];
				break;
			case 'q':
				tcsetattr(0, TCSANOW, &oldtio);
				exit(0);
			default:
				break;
		}
	}
	return 0;
}
