#define _GNU_SOURCE
#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <glib.h>
#include <sys/select.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include "srcp.h"
#include "crcf.h"
#include "uuid.h"

struct srcp *srcp_cmd;
struct srcp *srcp_info;

struct crcf_message {
	char *actor;
	char *method;
	char *attribute;
	int (*handle)(int reply_to, char *actor_id, char *attribute_value);
};

static int handler_locodb_list_loco(int reply_to, char *actor_id, char *attribute_value)
{
	struct crcf_id *entry = NULL;
	char *reply;
	int count = 0;
		
	while ((entry = crcf_db_iterate(entry))) {
		if (entry->actor != CRCF_ACTOR_LOCO)
			continue;
		count++;
	}
	asprintf(&reply, "LOCODB 1 INFO LOCOCOUNT %d", count);
		
	srcp_message_set(srcp_cmd, 
	    reply_to, srcp_session_id_get(srcp_info), "CRCF", reply);
	free(reply);
	
	while ((entry = crcf_db_iterate(entry))) {
		struct crcf_loco *loco;
		
		if (entry->actor != CRCF_ACTOR_LOCO)
			continue;
		loco = (struct crcf_loco *)entry;
	
		asprintf(&reply, "LOCODB 1 INFO LOCO %s", loco->id.id);
		
		srcp_message_set(srcp_cmd, 
		    reply_to, srcp_session_id_get(srcp_info), "CRCF", reply);

		free(reply);
	}

	return 0;
}

static int handler_loco_get_name(int reply_to, char *actor_id, char *attribute_value)
{
	struct crcf_loco *loco;
	char *reply;
	char *name;
	
	loco = (struct crcf_loco *)crcf_db_search_id_actor(actor_id, CRCF_ACTOR_LOCO);
	if (!loco)
		return -1;
		
	name = crcf_str_encode(loco->name);
	asprintf(&reply, "LOCO %s INFO NAME %s",
	    loco->id.id, name);
		
	srcp_message_set(srcp_cmd, 
	    reply_to, srcp_session_id_get(srcp_info), "CRCF", reply);

	free(name);
	free(reply);

	return 0;
}

static int handler_loco_list_decoder(int reply_to, char *actor_id, char *attribute_value)
{
	struct crcf_loco *loco;
	char *reply;
	
	loco = (struct crcf_loco *)crcf_db_search_id_actor(actor_id, CRCF_ACTOR_LOCO);
	if (!loco)
		return -1;

	asprintf(&reply, "LOCO %s INFO DECODERCOUNT 1", loco->id.id);

	srcp_message_set(srcp_cmd, 
	    reply_to, srcp_session_id_get(srcp_info), "CRCF", reply);
	free(reply);
		
	asprintf(&reply, "LOCO %s INFO DECODER %s",
	    loco->id.id, loco->decoder.id);
		
//	printf("reply: %s\n", reply);
	srcp_message_set(srcp_cmd, 
	    reply_to, srcp_session_id_get(srcp_info), "CRCF", reply);

	free(reply);

	return 0;
}

static int handler_decoder_list_protocol(int reply_to, char *actor_id, char *attribute_value)
{
	struct crcf_decoder *decoder;
	char *reply;
	
	decoder = (struct crcf_decoder *)crcf_db_search_id_actor(actor_id, CRCF_ACTOR_DECODER);
	if (!decoder)
		return -1;

	asprintf(&reply, "DECODER %s INFO PROTOCOLCOUNT 1", decoder->id.id);

	srcp_message_set(srcp_cmd, 
	    reply_to, srcp_session_id_get(srcp_info), "CRCF", reply);
	free(reply);
		
	asprintf(&reply, "DECODER %s INFO PROTOCOL %s",
	    decoder->id.id, decoder->protocol.id);
		
//	printf("reply: %s\n", reply);
	srcp_message_set(srcp_cmd, 
	    reply_to, srcp_session_id_get(srcp_info), "CRCF", reply);

	free(reply);

	return 0;
}

static int handler_protocol_get_address(int reply_to, char *actor_id, char *attribute_value)
{
	struct crcf_protocol *protocol;
	char *reply;
	
	protocol = (struct crcf_protocol *)crcf_db_search_id_actor(actor_id, CRCF_ACTOR_PROTOCOL);
	if (!protocol)
		return -1;
		
	asprintf(&reply, "PROTOCOL %s INFO ADDRESS %d",
	    protocol->id.id, protocol->address);
		
	srcp_message_set(srcp_cmd, 
	    reply_to, srcp_session_id_get(srcp_info), "CRCF", reply);

	free(reply);

	return 0;
}

static int handler_protocol_get_bus(int reply_to, char *actor_id, char *attribute_value)
{
	struct crcf_protocol *protocol;
	char *reply;
	
	protocol = (struct crcf_protocol *)crcf_db_search_id_actor(actor_id, CRCF_ACTOR_PROTOCOL);
	if (!protocol)
		return -1;
		
	asprintf(&reply, "PROTOCOL %s INFO BUS %d",
	    protocol->id.id, protocol->bus);
		
	srcp_message_set(srcp_cmd, 
	    reply_to, srcp_session_id_get(srcp_info), "CRCF", reply);

	free(reply);

	return 0;
}

static int handler_protocol_get_speedsteps(int reply_to, char *actor_id, char *attribute_value)
{
	struct crcf_protocol *protocol;
	char *reply;
	
	protocol = (struct crcf_protocol *)crcf_db_search_id_actor(actor_id, CRCF_ACTOR_PROTOCOL);
	if (!protocol)
		return -1;
		
	asprintf(&reply, "PROTOCOL %s INFO SPEEDSTEPS %d",
	    protocol->id.id, protocol->speedsteps);
		
	srcp_message_set(srcp_cmd, 
	    reply_to, srcp_session_id_get(srcp_info), "CRCF", reply);

	free(reply);

	return 0;
}

static int handler_protocol_list_function(int reply_to, char *actor_id, char *attribute_value)
{
	struct crcf_protocol *protocol;
	char *reply;
	int functioncount;
	struct crcf_function *function = NULL;
	
	protocol = (struct crcf_protocol *)crcf_db_search_id_actor(actor_id, CRCF_ACTOR_PROTOCOL);
	if (!protocol)
		return -1;

	functioncount = crcf_db_search_protocol_functioncount(protocol);
		
	asprintf(&reply, "PROTOCOL %s INFO FUNCTIONCOUNT %d",
	    protocol->id.id, functioncount);
	srcp_message_set(srcp_cmd, 
	    reply_to, srcp_session_id_get(srcp_info), "CRCF", reply);
	free(reply);
	
	while ((function = crcf_db_iterate_protocol_function(protocol, function))) {
		asprintf(&reply, "PROTOCOL %s INFO FUNCTION %s",
		    protocol->id.id, function->id.id);
		srcp_message_set(srcp_cmd, 
		    reply_to, srcp_session_id_get(srcp_info), "CRCF", reply);
		free(reply);
	}

	return 0;
}

static int handler_function_get_number(int reply_to, char *actor_id, char *attribute_value)
{
	struct crcf_function *function;
	char *reply;
	
	function = (struct crcf_function *)crcf_db_search_id_actor(actor_id, CRCF_ACTOR_FUNCTION);
	if (!function)
		return -1;
		
	asprintf(&reply, "FUNCTION %s INFO NUMBER %d",
	    function->id.id, crcf_function_number_get(function));
		
	srcp_message_set(srcp_cmd, 
	    reply_to, srcp_session_id_get(srcp_info), "CRCF", reply);

	free(reply);

	return 0;
}

static int handler_function_get_mode(int reply_to, char *actor_id, char *attribute_value)
{
	struct crcf_function *function;
	char *reply;
	
	function = (struct crcf_function *)crcf_db_search_id_actor(actor_id, CRCF_ACTOR_FUNCTION);
	if (!function)
		return -1;
		
	asprintf(&reply, "FUNCTION %s INFO MODE %d",
	    function->id.id, crcf_function_mode_get(function));
		
	srcp_message_set(srcp_cmd, 
	    reply_to, srcp_session_id_get(srcp_info), "CRCF", reply);

	free(reply);

	return 0;
}

static struct crcf_message handlers[] = {
	{ "LOCODB",   "LIST", "LOCO",       handler_locodb_list_loco },
	{ "LOCO",     "GET",  "NAME",       handler_loco_get_name },
	{ "LOCO",     "LIST", "DECODER",    handler_loco_list_decoder },
	{ "DECODER",  "LIST", "PROTOCOL",   handler_decoder_list_protocol },
	{ "PROTOCOL", "GET",  "ADDRESS",    handler_protocol_get_address },
	{ "PROTOCOL", "GET",  "BUS",        handler_protocol_get_bus },
	{ "PROTOCOL", "GET",  "SPEEDSTEPS", handler_protocol_get_speedsteps },
	{ "PROTOCOL", "LIST", "FUNCTION",   handler_protocol_list_function },
	{ "FUNCTION", "GET",  "NUMBER",     handler_function_get_number },
	{ "FUNCTION", "GET",  "MODE",       handler_function_get_mode },
};

static void message_hook(struct srcp *srcp, int send_to, int reply_to, char *type, char *msg)
{
//	printf("Message send_to: %d, reply_to: %d, type: %s, msg: %s\n",
//	    send_to, reply_to, type, msg);

	if (!strcmp(type, "CRCF")) {
		char *actor, *actor_id, *method, *attribute, *attribute_value;
		int i;
		
		sscanf(msg, "%ms %ms %ms %ms %ms", 
		    &actor, &actor_id, &method, &attribute, & attribute_value);
//		printf("%s %s %s %s %s\n", actor, actor_id, method, attribute, attribute_value);
		
		for (i = 0; i < sizeof(handlers)/sizeof(struct crcf_message); i++) {
			if (!strcmp(actor, handlers[i].actor) &&
			    !strcmp(method, handlers[i].method) &&
			    !strcmp(attribute, handlers[i].attribute)) {
				handlers[i].handle(reply_to, actor_id, attribute_value);
				break;
			}
		}
		
		free(actor);
		free(actor_id);
		free(method);
		free(attribute);
		free(attribute_value);
	}
}

int main(int argc, char **argv)
{
	GKeyFile *keyfile;
	gboolean loaded;
	gchar *keyfilename = "srcp-db.db";
	struct srcp_loc_dcc *decoder = NULL;
	int decoders_nr = 0;
	int fd_srcp_info;
	int fd_max = 0;
	bool save = false;
	
	if (argc < 2) {
		printf("Usage: %s [host]\n", argv[0]);
		return 1;
	}
	
	srcp_cmd = srcp_create(argv[1], "4303");
	if (!srcp_cmd) {
		perror("srcp_create()");
		return 1;
	}
	srcp_go(srcp_cmd);

	keyfile = g_key_file_new();
	
	loaded = g_key_file_load_from_file(
	    keyfile, 
	    keyfilename,
	    G_KEY_FILE_KEEP_COMMENTS | G_KEY_FILE_KEEP_TRANSLATIONS,
	    NULL);

	if (loaded) {
		gsize groups_nr;
		gchar **groups;
		int i;
		
		printf("Database file loaded\n");
		
		groups = g_key_file_get_groups(keyfile, &groups_nr);
		printf("Loaded %d decoders\n", (int)groups_nr);
		
		for (i = 0; i < groups_nr; i++) {
			struct crcf_loco *crcf_loco;
			struct crcf_decoder *crcf_decoder;
			struct crcf_protocol *crcf_protocol;
			char *name = "unnamed";
			char *loco_id = NULL;
			char *decoder_id = NULL;
			char *protocol_id = NULL;
			int j;
			
			printf("Decoder: %s\n", groups[i]);
			
			decoders_nr++;
			decoder = calloc(sizeof(struct srcp_loc_dcc), 1);
			
			memset(decoder, 0, sizeof(struct srcp_loc_dcc));
			decoder->address = atoi(groups[i]);

			if (decoder->address < 128) {
				decoder->protocolversion = SRCP_LOC_DCC_PROTOCOLVERSION_SHORT;
			} else {
				decoder->protocolversion = SRCP_LOC_DCC_PROTOCOLVERSION_LONG;
			}

			if (g_key_file_has_key(keyfile, groups[i], "functions", NULL)) {
				decoder->nrofdecoderfunctions =
				    g_key_file_get_integer(keyfile, groups[i], "functions", NULL);
			} else
				decoder->nrofdecoderfunctions = 5;
			if (decoder->nrofdecoderfunctions > SRCP_LOC_DCC_FUNCTIONS) {
				decoder->nrofdecoderfunctions = SRCP_LOC_DCC_FUNCTIONS;
				printf("Maximum number of functions: %d",
				    SRCP_LOC_DCC_FUNCTIONS);
			}
			if (decoder->nrofdecoderfunctions) {
				/* If possible switch on light */
				decoder->functions[0] = true;
			}
			
			if (g_key_file_has_key(keyfile, groups[i], "bus", NULL)) {
				decoder->bus =
				    g_key_file_get_integer(keyfile, groups[i], "bus", NULL);
			} else
				decoder->bus = 1;

			if (g_key_file_has_key(keyfile, groups[i], "speedsteps", NULL)) {
				decoder->decoderspeedsteps =
				    g_key_file_get_integer(keyfile, groups[i], "speedsteps", NULL);
			} else {
				decoder->decoderspeedsteps = 128;
			}
			if (decoder->decoderspeedsteps == 128) {
				decoder->V_max = 126;
			} else {
				decoder->V_max = decoder->decoderspeedsteps;
			}
			decoder->drivemode = SRCP_LOC_DCC_DRIVEMODE_FORWARD;

			if (g_key_file_has_key(keyfile, groups[i], "name", NULL)) {
				name =
				    g_key_file_get_string(keyfile, groups[i], "name", NULL);
				printf("Name: %s\n", name);
			}

			if (g_key_file_has_key(keyfile, groups[i], "active", NULL)) {
				bool active =
				    g_key_file_get_boolean(keyfile, groups[i], "active", NULL);

				if (active)
					srcp_loc_dcc_init(srcp_cmd, decoder);
			}

			if (g_key_file_has_key(keyfile, groups[i], "loco_id", NULL)) {
				loco_id =
				    g_key_file_get_string(keyfile, groups[i], "loco_id", NULL);
			} else {
				printf("missing loco_id for %s\n", name);
				loco_id = uuid_v4_create_alloc();
				g_key_file_set_string(keyfile, groups[i], "loco_id", loco_id);
				save = true;
			}
			if (g_key_file_has_key(keyfile, groups[i], "decoder_id", NULL)) {
				decoder_id =
				    g_key_file_get_string(keyfile, groups[i], "decoder_id", NULL);
			} else {
				printf("missing decoder_id for %s\n", name);
				decoder_id = uuid_v4_create_alloc();
				g_key_file_set_string(keyfile, groups[i], "decoder_id", decoder_id);
				save = true;
			}
			if (g_key_file_has_key(keyfile, groups[i], "protocol_id", NULL)) {
				protocol_id =
				    g_key_file_get_string(keyfile, groups[i], "protocol_id", NULL);
			} else {
				printf("missing protocol_id for %s\n", name);
				protocol_id = uuid_v4_create_alloc();
				g_key_file_set_string(keyfile, groups[i], "protocol_id", protocol_id);
				save = true;
			}
			
			crcf_loco = crcf_loco_create(loco_id);
			decoder->private = crcf_loco;
			crcf_loco_name_set(crcf_loco, name);
			
			crcf_decoder = crcf_decoder_create(decoder_id);
			crcf_loco_decoder_set(crcf_loco, crcf_decoder);
			crcf_decoder_loco_set(crcf_decoder, crcf_loco);
			
			crcf_protocol = crcf_protocol_create(protocol_id);
			crcf_decoder_protocol_set(crcf_decoder, crcf_protocol);
			crcf_protocol_address_set(crcf_protocol, decoder->address);
			crcf_protocol_bus_set(crcf_protocol, decoder->bus);
			crcf_protocol_decoder_set(crcf_protocol, crcf_decoder);

			for (j = 0; j < decoder->nrofdecoderfunctions; j++) {
				char key[100];
				char *f_id = NULL;
				bool mode = 0;
				struct crcf_function *crcf_function;
				
				sprintf(key, "f%d_id", j);
				if (g_key_file_has_key(keyfile, groups[i], key, NULL)) {
					f_id = g_key_file_get_string(keyfile, groups[i], key, NULL);
				} else {
					printf("missing function_id for %s\n", name);
					f_id = uuid_v4_create_alloc();
					g_key_file_set_string(keyfile, groups[i], key, f_id);
					save = true;
				}
				sprintf(key, "f%d_push", j);
				if (g_key_file_has_key(keyfile, groups[i], key, NULL)) {
					mode = 
					    g_key_file_get_boolean(keyfile, groups[i], key, NULL);
				}
				crcf_function = crcf_function_create(f_id);
				crcf_function_number_set(crcf_function, j);
				crcf_function_mode_set(crcf_function, mode);
				crcf_function_protocol_set(crcf_function, crcf_protocol);
			}
		}
		
		g_strfreev(groups);
	} else {
		printf("Database file not loaded\n");
	}

	if (save) {
		gsize len;
		char *newfile = g_key_file_to_data (keyfile, &len, NULL);
		int fdkf;

		fdkf = open(keyfilename, O_WRONLY);

		printf("Rewritting file with new ids\n");
		
		write(fdkf, newfile, len);
		close(fdkf);
		
		free(newfile);
	}
	
	g_key_file_free(keyfile);

	printf("Opening info connection to %s\n", argv[1]);
	srcp_info = srcp_create(argv[1], "4303");
	if (!srcp_info) {
		perror("srcp_create()");
		return 1;
	}
	srcp_connectionmode_set(srcp_info, SRCP_CONNECTIONMODE_INFO);

	srcp_go(srcp_info);
	fd_srcp_info = srcp_fd_get(srcp_info);

	srcp_hook_message(srcp_info, message_hook);
	

	fd_max = 1;
	if (fd_srcp_info >= fd_max)
		fd_max = fd_srcp_info + 1;

	while (1) {
		fd_set read_set;

		FD_ZERO(&read_set);
		FD_SET(fd_srcp_info, &read_set);

		select(fd_max, &read_set, NULL, NULL, NULL);

		if (FD_ISSET(fd_srcp_info, &read_set)) {
			if (srcp_info_handle(srcp_info) == SRCP_RESPONSE_ERROR_NO_DATA) {
				return -1;
			}

		}
	}
	
	return 0;
}
