#include <stdio.h>
#include <stdlib.h>
#include <termios.h>
#include <unistd.h>
#include <string.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include "srcp.h"
#include <linux/joystick.h>
#include <glib.h>
#include <errno.h>

struct srcp *srcp;

struct switch_decoder {
	struct srcp_accessory_dcc decoder;
	int straight;
	int curved;
};

struct switch_decoder *switch_decoders = NULL;
int switch_decoder_nr = 0;


int joystick_event(int fd)
{
	struct js_event event;
	int ret, i;
	
	do {
		ret = read (fd, &event, sizeof(struct js_event));
		if (ret == sizeof(struct js_event)) {
			if (event.type & JS_EVENT_BUTTON) {
				for (i = 0; i < switch_decoder_nr; i++) {
					if (event.number == switch_decoders[i].straight) {
						printf("decoder %d port 0: %d\n",
						     switch_decoders[i].decoder.address,
						     event.value);
						srcp_accessory_dcc_set(srcp, 
						    &switch_decoders[i].decoder,
						    0, event.value);
					}
					if (event.number == switch_decoders[i].curved) {
						printf("decoder %d port 1: %d\n",
						     switch_decoders[i].decoder.address,
						     event.value);
						srcp_accessory_dcc_set(srcp, 
						    &switch_decoders[i].decoder,
						    1, event.value);
					}
				}
			}
		}
	} while (ret == sizeof(struct js_event));
	if (ret <= 0 && errno != EAGAIN) {
		perror("Device read failed");
		return -1;
	}
	return 0;
}


int main(int argc, char **argv)
{
	char *joystick_device = NULL;
	char *host = NULL;
	int fd_joystick;
	char nr_axes;
	char nr_buttons;
	int fd_max = 0;
	GKeyFile *keyfile;
	gboolean loaded;
	gchar *keyfilename;
	
	if (argc < 2) {
		printf("Usage %s [config_file]\n", 
		    argv[0]);
		return -1;
	}
	
	keyfilename = argv[1];
	
	keyfile = g_key_file_new();
	
	loaded = g_key_file_load_from_file(
	    keyfile, 
	    keyfilename,
	    G_KEY_FILE_NONE,
	    NULL);

	if (loaded) {
		gsize groups_nr;
		gchar **groups;
		int i;
		
		printf("Config file loaded\n");
		
		groups = g_key_file_get_groups(keyfile, &groups_nr);
		
		for (i = 0; i < groups_nr; i++) {
			if (!strcmp(groups[i], "global")) {
				if (g_key_file_has_key(keyfile, groups[i], "host", NULL)) {
					host = g_key_file_get_string(
					    keyfile, groups[i], "host", NULL);
				}
				if (g_key_file_has_key(keyfile, groups[i], "joystick", NULL)) {
					joystick_device = g_key_file_get_string(
					    keyfile, groups[i], "joystick", NULL);
				}
				printf("Opening connection to %s\n", host);
	
				srcp = srcp_create(host, "4303");
				if (!srcp) {
					perror("srcp_create()");
					return 1;
				}

				srcp_connectionmode_set(srcp, SRCP_CONNECTIONMODE_COMMAND);
				srcp_go(srcp);

			}
			if (!strncmp(groups[i], "switch", strlen("switch"))) {
				if (g_key_file_has_key(keyfile, groups[i], "address", NULL) &&
				    g_key_file_has_key(keyfile, groups[i], "straight", NULL) &&
				    g_key_file_has_key(keyfile, groups[i], "curved", NULL) ) {
					int address;
					int straight;
					int curved;
					int response;
					
					switch_decoders = realloc(
					    switch_decoders,
					    sizeof(struct switch_decoder) * (switch_decoder_nr+1));

					address = g_key_file_get_integer(
					    keyfile, groups[i], "address", NULL);
					straight = g_key_file_get_integer(
					    keyfile, groups[i], "straight", NULL);
					curved = g_key_file_get_integer(
					    keyfile, groups[i], "curved", NULL);

					switch_decoders[switch_decoder_nr].decoder.address = address;
					switch_decoders[switch_decoder_nr].decoder.bus = 1;
					switch_decoders[switch_decoder_nr].decoder.port[0] = false;
					switch_decoders[switch_decoder_nr].decoder.port[1] = false;
					switch_decoders[switch_decoder_nr].straight = straight;
					switch_decoders[switch_decoder_nr].curved = curved;
					
					printf("Switch %s: address %d, buttons %d and %d: ",
					    groups[i], address, straight, curved);
					response = srcp_accessory_dcc_get(srcp,
					    &switch_decoders[switch_decoder_nr].decoder);
					if (SRCP_RESPONSE_ERROR(response)) {
						srcp_accessory_dcc_init(srcp,
						    &switch_decoders[switch_decoder_nr].decoder);
						printf("created\n");
					} else {
						printf("exists\n");
					}
					switch_decoder_nr++;
				}
			}
		}
		
		g_strfreev(groups);
	} else {
		printf("Config file not loaded\n");
		return 1;
	}

	if (!host || !joystick_device) {
		printf("No hostname of joystick device given\n");
		return 1;
	}
	
	g_key_file_free(keyfile);

	printf("Joystick device: %s\n", joystick_device);
	fd_joystick = open(joystick_device, O_RDONLY);
	if (fd_joystick < 0) {
		printf("Could not open device %s\n", joystick_device);
		printf("Going to simulate a joystick\n");
//		return -1;
	} else {
		ioctl(fd_joystick, FIONBIO, &(int){1});
		ioctl(fd_joystick, JSIOCGAXES, &nr_axes);
		ioctl(fd_joystick, JSIOCGBUTTONS, &nr_buttons);
	}
	printf("%s: %d axes, %d buttons\n", joystick_device, nr_axes, nr_buttons);
	
	
	fd_max = fd_joystick + 1;
	
	while (1) {
		fd_set read_set;
		
		FD_ZERO(&read_set);
		FD_SET(fd_joystick, &read_set);

		select(fd_max, &read_set, NULL, NULL, NULL);

		if (fd_joystick >= 0 && FD_ISSET(fd_joystick, &read_set)) {
			if (joystick_event(fd_joystick)) {
				close(fd_joystick);
				return 1;
			}
		}
	}
	return 0;
}
