#ifndef _INCLUDE_SRCP_H_
#define _INCLUDE_SRCP_H_

#include <stdbool.h>

struct srcp;

struct srcp *srcp_create(char *host, char *port);

#define SRCP_LOC_DCC_FUNCTIONS 29

#define SRCP_LOC_DCC_PROTOCOLVERSION_SHORT	1
#define SRCP_LOC_DCC_PROTOCOLVERSION_LONG	2

#define SRCP_LOC_DCC_DRIVEMODE_BACKWARD		0
#define SRCP_LOC_DCC_DRIVEMODE_FORWARD		1
#define SRCP_LOC_DCC_DRIVEMODE_EMERGENCY_STOP	2

struct srcp_loc_dcc {
	int bus;
	int address;
	int protocolversion;
	int decoderspeedsteps;
	int nrofdecoderfunctions;
	int drivemode;
	int V;
	int V_max;
	bool functions[SRCP_LOC_DCC_FUNCTIONS];
	void *private;

	struct srcp_loc_dcc *next;
	int refcount;
	
	int lock;
};

struct srcp_accessory_dcc {
	int bus;
	int address;

	bool port[2];
	void *private;

	struct srcp_accessory_dcc *next;
	int refcount;

	int lock;
};

enum srcp_connectionmode {
	SRCP_CONNECTIONMODE_INFO,
	SRCP_CONNECTIONMODE_COMMAND,
};

int srcp_connectionmode_set(struct srcp *srcp, enum srcp_connectionmode mode);
int srcp_go(struct srcp *srcp);

int srcp_session_id_get(struct srcp *srcp);

int srcp_fd_get(struct srcp *srcp);

int srcp_hook_all(struct srcp *srcp,
    void(*all_hook)(struct srcp *srcp, char *message));

int srcp_hook_loc_dcc_init(struct srcp *srcp, 
    void(*loc_dcc_init_hook)(struct srcp *srcp, struct srcp_loc_dcc *loc));
int srcp_hook_loc_dcc(struct srcp *srcp, 
    void(*loc_dcc_hook)(struct srcp *srcp, struct srcp_loc_dcc *loc));

int srcp_hook_lock_loc_dcc(struct srcp *srcp,
    void(*lock_loc_dcc_hook)(struct srcp *srcp, struct srcp_loc_dcc *loc));

int srcp_hook_accessory_dcc_init(struct srcp *srcp, 
    void(*accessory_dcc_init_hook)(struct srcp *srcp, 
    struct srcp_accessory_dcc *accessory));
int srcp_hook_accessory_dcc(struct srcp *srcp, 
    void(*accessory_dcc_hook)(struct srcp *srcp, 
    struct srcp_accessory_dcc *accessory));

int srcp_hook_power(struct srcp *srcp, 
    void(*power_hook)(struct srcp *srcp, bool power));

int srcp_hook_message(struct srcp *srcp,
    void (*message_hook)(struct srcp *srcp, int send_to, int reply_to, char *type, char *msg));

int srcp_loc_dcc_init(struct srcp *srcp, struct srcp_loc_dcc *loc);
int srcp_loc_dcc_get(struct srcp *srcp, struct srcp_loc_dcc *loc);
int srcp_loc_dcc_set(struct srcp *srcp, struct srcp_loc_dcc *loc);
int srcp_loc_dcc_term(struct srcp *srcp, struct srcp_loc_dcc *loc);
void srcp_loc_dcc_list_add(struct srcp *srcp, struct srcp_loc_dcc *loc);
struct srcp_loc_dcc *srcp_loc_dcc_list_next(struct srcp *srcp, struct srcp_loc_dcc *loc);
struct srcp_loc_dcc *srcp_loc_dcc_list_prev(struct srcp *srcp, struct srcp_loc_dcc *loc);

int srcp_lock_loc_dcc_set(struct srcp *srcp, struct srcp_loc_dcc *dcc);
int srcp_lock_loc_dcc_term(struct srcp *srcp, struct srcp_loc_dcc *dcc);

int srcp_accessory_dcc_init(struct srcp *srcp, struct srcp_accessory_dcc *acc);
int srcp_accessory_dcc_get(struct srcp *srcp, struct srcp_accessory_dcc *acc);
int srcp_accessory_dcc_set(struct srcp *srcp, struct srcp_accessory_dcc *acc, int port, bool value);
int srcp_accessory_dcc_term(struct srcp *srcp, struct srcp_accessory_dcc *acc);

int srcp_power_set(struct srcp *srcp, int bus, bool value);
int srcp_power_get(struct srcp *srcp, int bus, bool *value);

int srcp_sm_init(struct srcp *srcp, int bus);
int srcp_sm_term(struct srcp *srcp, int bus);
int srcp_sm_set_cv(struct srcp *srcp, int bus, int address, int cv, int value);

int srcp_message_set(struct srcp *srcp, int send_to, int reply_to, char *type, char *msg);

int srcp_info_handle(struct srcp *srcp);
void srcp_loc_dcc_put(struct srcp_loc_dcc *loc);
void srcp_loc_dcc_take(struct srcp_loc_dcc *loc);

#define SRCP_RESPONSE_INFO	100
#define SRCP_RESPONSE_INFO_INIT	101
#define SRCP_RESPONSE_INFO_TERM	102
#define SRCP_RESPONSE_OK	200

#define SRCP_RESPONSE_ERROR(resp) ((resp) >= 400)

#define SRCP_RESPONSE_ERROR_NO_DATA	416

#endif /* _INCLUDE_SRCP_H_ */
