/*
	Copyright Jeroen Vreeken (jeroen@vreeken.net), 2014

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */

#include "uuid.h"
#include <stdint.h>
#include <unistd.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <string.h>
#include <sys/ioctl.h>
#include <malloc.h>

int uuid_v4_create(char uuid[37])
{
	uint8_t octet[16];
	int fdr;
	ssize_t r, off;
	int retval = 0;
	
	memset(octet, 0xff, sizeof(octet));
	
	fdr = open("/dev/random", O_RDONLY);
	
	if (fdr < 0)
		return -1;
	
	ioctl(fdr, FIONBIO, &(int){0});

	for (off = 0; off < sizeof(octet); ) {
		r = read(fdr, octet + off, sizeof(octet) - off);
		
		if (r <= 0) {
			retval = -1;
			break;
		} else {
			off += r;
		}
	}

	octet[6] = (octet[6] & 0x0f) | 0x40;
	octet[8] = (octet[8] & 0x3f) | 0x80;
	
	sprintf(uuid, 
	    "%02x%02x%02x%02x-"
	    "%02x%02x-%02x%02x-%02x%02x-"
	    "%02x%02x%02x%02x%02x%02x",
	    octet[0], octet[1], octet[2], octet[3],
	    octet[4], octet[5],
	    octet[6], octet[7],
	    octet[8], octet[9],
	    octet[10], octet[11], octet[12], octet[13], octet[14], octet[15]);
	
	return retval;
}

char * uuid_v4_create_alloc(void)
{
	char *id;
	
	id = malloc(37);
	if (!id)
		return id;
	
	if (uuid_v4_create(id)) {
		free(id);
		return NULL;
	}
	
	return id;
}
